(* file: sun.ml
 * author: Bob Muller
 * date: October 8, 2015
 * revised: September, 2016
 *
 * CS1103 Computer Science I Honors
 *
 * A simple animation example, a setting or rising sun over water.
 * Click to change setting to rising and vice-versa.
 *
 * To run it:
 * > make
 * > ./go
 *
 *)
open World
open Image
open Color
(* open Cs1103 *)

let displayWidth = 600.
let displayHeight = displayWidth

let radius = displayWidth /. 6.0
let dropAmount = 5.0

let sky   = Image.rectangle displayWidth displayHeight Color.lightBlue
let water = Image.rectangle displayWidth (displayHeight /. 2.0) Color.blue
let sun   = Image.circle radius Color.orange

(* What is the state of the sun? 
*)
type state = Up | Setting | Down | Rising

type model = {state : state; y : float}

(* view : model -> Image.t
*)
let view ({state; y} : model) =
  let x = displayWidth /. 2.0 -. radius in
  let sky = Image.place_image sun (x, y -. radius) sky
  in
  Image.place_image water (0.0, displayHeight -. radius) sky

(* update : model -> model World.t
*)
let update ({state; y} as model : model) =
  match state with
  | Setting -> let newY = min displayHeight (y +. dropAmount)
               in
               World { model with y = newY}
          
  | Rising -> let newY = max 0.0 (y -. dropAmount)
              in
              World { model with y = newY}
          
  | _ -> World model

(* handleMouse : model -> float -> float -> event -> World.t
*)
let handleMouse ({state; y} as model) x y event =
  match (event, state) with
  | ("button_down", Up)      -> World { model with state = Setting}
  | ("button_down", Setting) -> World { model with state = Rising}
  | ("button_down", Down)    -> World { model with state = Rising}
  | ("button_down", Rising)  -> World { model with state = Setting}
  | _                        -> World model

let initialModel : model = {state = Up; y = 0.0}

let _ =
  World.big_bang
    initialModel
    ~name: "Setting/Rising Sun"
		~width: (int_of_float displayWidth)
		~height: (int_of_float displayHeight)
		~to_draw: view
		~rate: 0.05
    ~on_tick: update
    ~on_mouse: handleMouse
